@@ -2,7 +2,7 @@
  * Memory handling
  * Library: joedog
  * 
- * Copyright (C) 2000-2007 by
+ * Copyright (C) 2000-2013 by
  * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
@@ -26,6 +26,7 @@
 #include <memory.h>
 #include <notify.h>
 #include <string.h>
+#include <stdarg.h>
 
 char *
 xstrdup(const char *str)
@@ -53,6 +54,41 @@ xstrdup(const char *str)
   return ret; 
 }
 
+char *
+xstrcat(const char *arg1, ...)
+{
+  const char *argptr;
+  char *resptr, *result;
+  int nargs = 0;
+  size_t  len = 0;
+  va_list valist;
+
+  va_start(valist, arg1);
+
+  for(argptr = arg1; argptr != NULL; argptr = va_arg(valist, char *))
+    len += strlen(argptr);
+
+  va_end(valist);
+
+  result = xmalloc(len + 1);
+  resptr = result;
+
+  va_start(valist, arg1);
+
+  nargs = 0;
+  for(argptr = arg1; argptr != NULL; argptr = va_arg(valist, char *)) {
+    len = strlen(argptr);
+    memcpy(resptr, argptr, len);
+    resptr += len;
+  }
+
+  va_end(valist);
+
+  *resptr = '\0';
+
+  return result;
+}
+
 /**
  * xrealloc: value added realloc
  */
