@@ -38,7 +38,6 @@
 
 #include <auth.h>
 
-
 struct trans
 {
   int   bytes;
@@ -57,13 +56,13 @@ typedef struct
   unsigned int   code;
   unsigned int   fail;
   unsigned int   ok200;
-  URL    **U;
+  ARRAY  urls;
   struct {
-    DIGEST_CHLG *wwwchlg;
-    DIGEST_CRED *wwwcred;
-    int  www;
-    DIGEST_CHLG *proxychlg;
-    DIGEST_CRED *proxycred;
+    DCHLG *wchlg;
+    DCRED *wcred;
+    int    www;
+    DCHLG *pchlg;
+    DCRED *pcred;
     int  proxy;
     struct {
       int  www;
@@ -77,8 +76,8 @@ typedef struct
   int      status;
   float    time;
   unsigned int rand_r_SEED;
-  float bigtime;
-  float smalltime;
+  float himark;
+  float lomark;
 } CLIENT;
 
 void * start_routine(CLIENT *client);
