@@ -27,15 +27,17 @@
 # include <unistd.h>
 #endif
 
+#include <joedog/boolean.h>
+
 typedef struct HASH_T *HASH;
 
-HASH   new_hash(ssize_t size);
-void   hash_add(HASH this, char *key, char *value);
-char  *hash_get(HASH this, char *key);
-char **hash_get_keys(HASH this);
-int    hash_lookup(HASH this, char *key);
-void   hash_destroy(HASH this);
-void   hash_free_keys(HASH this, char **keys);
-int    hash_get_entries(HASH this);
+HASH     new_hash();
+void     hash_add(HASH this, char *key, char *value);
+char *   hash_get(HASH this, char *key);
+char **  hash_get_keys(HASH this);
+BOOLEAN  hash_lookup(HASH this, char *key);
+void     hash_destroy(HASH this);
+void     hash_free_keys(HASH this, char **keys);
+int      hash_get_entries(HASH this);
 
 #endif/*HASH_H*/
