@@ -74,10 +74,10 @@ struct headers
 } HEADERS;
 
 /* http function prototypes */
-int      http_get (CONN *C, URL *U);
-int      http_post(CONN *C, URL *U);
+BOOLEAN  http_get (CONN *C, URL U);
+BOOLEAN  http_post(CONN *C, URL U);
 void     http_free_headers(HEADERS *h);
-HEADERS *http_read_headers(CONN *C, URL *U);
+HEADERS *http_read_headers(CONN *C, URL U);
 ssize_t  http_read(CONN *C);
 BOOLEAN  https_tunnel_request(CONN *C, char *host, int port);
 int      https_tunnel_response(CONN *C);
