@@ -34,6 +34,10 @@
 #include <ctype.h>
 #include <stdarg.h>
 
+#ifdef  HAVE_LIMITS_H
+# include <limits.h>
+#endif/*HAVE_LIMITS_H*/
+
 #if HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
@@ -110,6 +114,7 @@ int strlen();
 #endif
 
 #include <url.h>
+#include <auth.h>
 #include <array.h>
 #include <joedog/joedog.h>
 #include <joedog/boolean.h>
@@ -151,6 +156,7 @@ struct CONFIG
   char    *url;         /* URL for the single hit invocation.      */
   char    logfile[128]; /* alternative user defined simbot.log     */ 
   BOOLEAN verbose;      /* boolean, verbose output to screen       */
+  BOOLEAN quiet;        /* boolean, turn off all output to screen  */
   BOOLEAN csv;          /* boolean, display verbose output in CSV  */
   BOOLEAN fullurl;      /* boolean, display full url in verbose    */
   BOOLEAN display;      /* boolean, display the thread id verbose  */
@@ -160,6 +166,7 @@ struct CONFIG
   int     timeout;      /* socket connection timeout value, def:10 */
   BOOLEAN bench;        /* signifies a benchmarking run, no delay  */
   BOOLEAN internet;     /* use random URL selection if TRUE        */
+  BOOLEAN timestamp;    /* timestamp the output                    */
   int     time;         /* length of the siege in hrs, mins, secs  */
   int     secs;         /* time value for the lenght of the siege  */
   int     reps;         /* reps to run the test, default infinite  */ 
@@ -167,6 +174,7 @@ struct CONFIG
   int     length;       /* length of the urls array, made global   */
   BOOLEAN debug;        /* boolean, undocumented debug command     */
   BOOLEAN chunked;      /* boolean, accept chunked encoding        */
+  BOOLEAN unique;       /* create unique files for upload          */
   BOOLEAN get;          /* get header information for debugging    */ 
   BOOLEAN mark;         /* signifies a log file mark req.          */ 
   char    *markstr;     /* user defined string value to mark file  */
@@ -174,25 +182,22 @@ struct CONFIG
   BOOLEAN cookies;      /* to use cookies or not to use cookies    */
   char uagent[256];     /* user defined User-Agent string.         */
   char encoding[256];   /* user defined Accept-Encoding string.    */
+  char conttype[256];   /* user defined default content type.      */
   char *username;       /* DEPRECATED!! */
   char *password;       /* DEPRECATED!! */ 
   int  bids;            /* W & P authorization bids before failure */
-  struct {
-    char *encode;
-    struct LOGIN *head;
-    pthread_mutex_t lock; 
-  } auth;  
+  AUTH auth;
   BOOLEAN keepalive;    /* boolean, connection keep-alive value    */
   int     signaled;     /* timed based testing notification bool.  */
   char    extra[2048];  /* extra http request headers              */ 
+  #if 0
   struct {
     BOOLEAN required;   /* boolean, TRUE == use a proxy server.    */
     char *hostname;     /* hostname for the proxy server.          */
     int  port;          /* port number for proxysrv                */ 
     char *encode;       /* base64 encoded username and password    */
-    struct LOGIN *head;
-    pthread_mutex_t lock; 
   } proxy;
+  #endif
   BOOLEAN login;        /* boolean, client must login first.       */
   char    *loginurl;    /* XXX: deprecated the initial login URL   */
   ARRAY   lurl;
@@ -208,6 +213,7 @@ struct CONFIG
   char    *ssl_cert;    /* PEM certificate file for client auth    */
   char    *ssl_key;     /* PEM private key file for client auth    */
   char    *ssl_ciphers; /* SSL chiphers to use : delimited         */ 
+  METHOD  method;       /* HTTP method for --get requests          */
   pthread_cond_t  cond;
   pthread_mutex_t lock;
 };
