@@ -1,7 +1,7 @@
 /**
  * SIEGE socket library
  *
- * Copyright (C) 2000-2009 by
+ * Copyright (C) 2000-2013 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -82,28 +82,39 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   struct hostent     *hp;
   char   hn[512];
   int    port;
-#if defined(_AIX)
+#if defined(__GLIBC__)
+  struct hostent hent;
+  char hbf[8192];
+#elif defined(sun)
+# ifndef HAVE_GETIPNODEBYNAME
+  struct hostent hent;
+  char hbf[8192];
+# endif/*HAVE_GETIPNODEBYNAME*/
+#elif defined(_AIX)
   char *aixbuf;
   int  rc;
-#endif/*_AIX*/ 
+#endif/*_AIX*/
 
-  C->encrypt = FALSE;
+  C->encrypt  = (C->scheme == HTTPS) ? TRUE: FALSE;
+  C->state    = UNDEF;
+  C->ftp.pasv = TRUE;
+  C->ftp.size = 0;
 
   memset(hn, '\0', sizeof hn);
  
   /* if we are using a proxy, then we make a socket
      connection to that server rather then a httpd */ 
-  if(my.proxy.required){
-    snprintf(hn, sizeof(hn), "%s", my.proxy.hostname);
-    port = my.proxy.port;
+  if (auth_get_proxy_required(my.auth)) {
+    snprintf(hn, sizeof(hn), "%s", auth_get_proxy_host(my.auth));
+    port = auth_get_proxy_port(my.auth);
   } else {
     snprintf(hn, sizeof(hn), "%s", hostparam);
     port = portparam;
   }
 
   /* create a socket, return -1 on failure */
-  if((C->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
-    switch(errno){
+  if ((C->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+    switch (errno) {
       case EPROTONOSUPPORT: { NOTIFY(ERROR, "unsupported protocol %s:%d",  __FILE__, __LINE__); break; }
       case EMFILE:          { NOTIFY(ERROR, "descriptor table full %s:%d", __FILE__, __LINE__); break; }
       case ENFILE:          { NOTIFY(ERROR, "file table full %s:%d",       __FILE__, __LINE__); break; }
@@ -112,17 +123,15 @@ new_socket(CONN *C, const char *hostparam, int portparam)
       default:              { NOTIFY(ERROR, "unknown socket error %s:%d",  __FILE__, __LINE__); break; }
     } socket_close(C); return -1;
   }
-  if(fcntl(C->sock, F_SETFD, O_NDELAY) < 0){
+  if (fcntl(C->sock, F_SETFD, O_NDELAY) < 0) {
     NOTIFY(ERROR, "unable to set close control %s:%d", __FILE__, __LINE__);
   }
 
 #if defined(__GLIBC__)
   {
-    struct hostent hent;
-    char hbf[8192]; 
     memset(hbf, '\0', sizeof hbf);
     /* for systems using GNU libc */
-    if((gethostbyname_r(hostparam, &hent, hbf, sizeof(hbf), &hp, &herrno) < 0)){
+    if ((gethostbyname_r(hostparam, &hent, hbf, sizeof(hbf), &hp, &herrno) < 0)) {
       hp = NULL;
     }
   }
@@ -131,8 +140,6 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   hp = getipnodebyname(hn, AF_INET, 0, &herrno);
 # else /* default use gethostbyname_r*/
   {
-    struct hostent hent;
-    char hbf[8192]; 
     memset(hbf, '\0', sizeof hbf);
     hp = gethostbyname_r(hn, &hent, hbf, sizeof(hbf), &herrno); 
   }
@@ -162,10 +169,10 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   cli.sin_family = AF_INET;
   cli.sin_port = htons(port);
 
-  if(C->connection.keepalive){
+  if (C->connection.keepalive) {
     opt = 1; 
-    if(setsockopt(C->sock,SOL_SOCKET,SO_KEEPALIVE,(char *)&opt,sizeof(opt))<0){
-      switch( errno ){
+    if (setsockopt(C->sock,SOL_SOCKET,SO_KEEPALIVE,(char *)&opt,sizeof(opt))<0) {
+      switch (errno) {
         case EBADF:       { NOTIFY(ERROR, "invalid descriptor %s:%d",    __FILE__, __LINE__); break; }
         case ENOTSOCK:    { NOTIFY(ERROR, "not a socket %s:%d",          __FILE__, __LINE__); break; }
         case ENOPROTOOPT: { NOTIFY(ERROR, "not a protocol option %s:%d", __FILE__, __LINE__); break; }
@@ -175,7 +182,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     }
   }
 
-  if((__socket_block(C->sock, FALSE)) < 0){
+  if ((__socket_block(C->sock, FALSE)) < 0) {
     NOTIFY(ERROR, "socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
     return -1; 
   }
@@ -187,8 +194,8 @@ new_socket(CONN *C, const char *hostparam, int portparam)
    */ 
   conn = connect(C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in));
   pthread_testcancel();
-  if(conn < 0 && errno != EINPROGRESS){
-    switch(errno){
+  if (conn < 0 && errno != EINPROGRESS) {
+    switch (errno) {
       case EACCES:        {NOTIFY(ERROR, "socket: %d EACCES",                  pthread_self()); break;}
       case EADDRNOTAVAIL: {NOTIFY(ERROR, "socket: %d address is unavailable.", pthread_self()); break;}
       case ETIMEDOUT:     {NOTIFY(ERROR, "socket: %d connection timed out.",   pthread_self()); break;}
@@ -209,7 +216,8 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     timeout.tv_sec  = (my.timeout > 0)?my.timeout:30;
     timeout.tv_usec = 0;
     res = select(C->sock+1, &rs, &ws, NULL, &timeout);
-    if(res < 1){
+    if ((res == -1) && (errno == EINTR)) {
+      pthread_testcancel();
       fprintf(stderr, "socket: connection timed out\n");
       socket_close(C);
       return -1; 
@@ -227,7 +235,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     }
   } /* end of connect conditional */
 
-  if((__socket_block(C->sock, TRUE)) < 0){
+  if ((__socket_block(C->sock, TRUE)) < 0) {
     NOTIFY(ERROR, "socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
     return -1; 
   }
@@ -246,9 +254,13 @@ __socket_check(CONN *C, SDSET mode)
   double timo;
   struct timeval timeout;
 
+  if (C->state == mode) {
+    return TRUE;
+  }
+
   FD_ZERO(&fds);
   FD_SET (C->sock, &fds);
-  if(mode==WRITE){
+  if (mode==WRITE) {
     *(&ws) = &fds;
   } else {
     *(&rs) = &fds;
@@ -258,28 +270,35 @@ __socket_check(CONN *C, SDSET mode)
   timeout.tv_sec  = (long)timo;
   timeout.tv_usec = 1000000L * (timo - (long)timo);
 
-  if(mode==WRITE){
+  if (mode==WRITE) {
     __socket_block(C->sock, FALSE);
   }
 
   do {
     res = select(C->sock + 1, rs, ws, NULL, &timeout);
     pthread_testcancel();
-  } while(res < 0 && errno == EINTR);
-
-  if(mode==WRITE){
+  } while (res < 0 && errno == EINTR);
+  
+  if (mode==WRITE) {
     __socket_block(C->sock, TRUE);
   }
 
-  if(res == 0){
+  if (res == 0) {
     errno = ETIMEDOUT;
   }
 
-  if(res < 1){
+  if (res < 1) {
     NOTIFY(WARNING, "socket: %d select timed out", pthread_self());
   }
 
-  return (res <= 0) ? FALSE : TRUE;
+  if (res <= 0) {
+    C->state = UNDEF;
+    return FALSE;
+  } else {
+    C->state = mode;
+    return TRUE;
+  }
+  //return (res <= 0) ? FALSE : TRUE;
 }
 
 /**
@@ -297,14 +316,14 @@ __socket_block(int sock, BOOLEAN block)
 #else 
   return sock;
 #endif
-
-  if(sock==-1){
+return sock;
+  if (sock==-1) {
     return sock;
   }
 
 #if HAVE_FCNTL_H 
-  if((flags = fcntl(sock, F_GETFL, 0)) < 0){
-    switch(errno){
+  if ((flags = fcntl(sock, F_GETFL, 0)) < 0) {
+    switch (errno) {
       case EACCES: { NOTIFY(ERROR, "EACCES %s:%d",                 __FILE__, __LINE__); break; }
       case EBADF:  { NOTIFY(ERROR, "bad file descriptor %s:%d",    __FILE__, __LINE__); break; }
       case EAGAIN: { NOTIFY(ERROR, "address is unavailable %s:%d", __FILE__, __LINE__); break; }
@@ -312,7 +331,7 @@ __socket_block(int sock, BOOLEAN block)
     } return -1;
   }
 
-  if(block){ 
+  if (block) { 
     flags &= ~O_NDELAY;
   } else {
     flags |=  O_NDELAY;
@@ -322,7 +341,7 @@ __socket_block(int sock, BOOLEAN block)
     #endif
   }
 
-  if((retval = fcntl(sock, F_SETFL, flags)) < 0){
+  if ((retval = fcntl(sock, F_SETFL, flags)) < 0) {
     NOTIFY(ERROR, "unable to set fcntl flags %s:%d", __FILE__, __LINE__);
     return -1;
   } 
@@ -339,7 +358,7 @@ __socket_block(int sock, BOOLEAN block)
  * writes vbuf to sock
  */
 private ssize_t
-__socket_write( int sock, const void *vbuf, size_t len )
+__socket_write(int sock, const void *vbuf, size_t len)
 {
   size_t      n;
   ssize_t     w;
@@ -347,9 +366,9 @@ __socket_write( int sock, const void *vbuf, size_t len )
  
   buf = vbuf;
   n   = len;
-  while(n > 0){
-    if((w = write( sock, buf, n)) <= 0){
-      if(errno == EINTR){
+  while (n > 0) {
+    if ((w = write( sock, buf, n)) <= 0) {
+      if (errno == EINTR) {
         w = 0;
       } else {
         return -1;
@@ -378,12 +397,12 @@ __ssl_socket_write(CONN *C, const void *vbuf, size_t len)
   buf = vbuf;
   n   = len;
 
-  while(n > 0){
-    if((w = SSL_write(C->ssl, buf, n)) <= 0){
-      if(w < 0) {
+  while (n > 0) {
+    if ((w = SSL_write(C->ssl, buf, n)) <= 0) {
+      if (w < 0) {
         err = SSL_get_error(C->ssl, w);
 
-        switch(err) {
+        switch (err) {
           case SSL_ERROR_WANT_READ:
           case SSL_ERROR_WANT_WRITE:
           return 0;
@@ -417,44 +436,44 @@ socket_read(CONN *C, void *vbuf, size_t len)
  
   buf = vbuf;
   n   = len;
-  if(C->encrypt == TRUE){
+  if (C->encrypt == TRUE) {
   #ifdef HAVE_SSL
-    while(n > 0){
-      if(__socket_check(C, READ) == FALSE){
+    while (n > 0) {
+      if (__socket_check(C, READ) == FALSE) {
 	return -1;
       }
-      if((r = SSL_read(C->ssl, buf, n)) < 0){
-        if( errno == EINTR )
+      if ((r = SSL_read(C->ssl, buf, n)) < 0) {
+        if (errno == EINTR)
           r = 0;
         else
           return -1;
       }
-      else if(r == 0) break;
+      else if (r == 0) break;
       n   -= r;
       buf += r;
     }   /* end of while    */
   #endif/*HAVE_SSL*/
   } else { 
-    while(n > 0){
-      if(C->inbuffer < len){
-        if(__socket_check(C, READ) == FALSE){
+    while (n > 0) {
+      if (C->inbuffer < len) {
+        if (__socket_check(C, READ) == FALSE) {
           return -1;
         }
       }
-      if(C->inbuffer <  n){
+      if (C->inbuffer <  n) {
         int lidos;
         memmove(C->buffer,&C->buffer[C->pos_ini],C->inbuffer);
         C->pos_ini = 0;
-	if(__socket_check(C, READ) == FALSE){
+	if (__socket_check(C, READ) == FALSE) {
 	  return -1;
 	}
         lidos = read(C->sock, &C->buffer[C->inbuffer], sizeof(C->buffer)-C->inbuffer);
-        if(lidos == 0)
+        if (lidos == 0)
           ret_eof = 1;
-        if(lidos < 0){
-          if(errno==EINTR || errno==EAGAIN)
+        if (lidos < 0) {
+          if (errno==EINTR || errno==EAGAIN)
             lidos = 0;
-          if(errno==EPIPE){
+          if (errno==EPIPE){
             return 0;
           } else {
             NOTIFY(ERROR, "socket: read error %s %s:%d", strerror(errno), __FILE__, __LINE__);
@@ -463,24 +482,24 @@ socket_read(CONN *C, void *vbuf, size_t len)
         }
         C->inbuffer += lidos;
       }
-      if(C->inbuffer >= n){
+      if (C->inbuffer >= n) {
         r = n;
       } else {
         r = C->inbuffer;
       }
-      if(r == 0) break;
+      if (r == 0) break;
       memmove(buf,&C->buffer[C->pos_ini],r);
       C->pos_ini  += r;
       C->inbuffer -= r;
       n   -= r;
       buf += r;
-      if(ret_eof) break;
+      if (ret_eof) break;
     } /* end of while */
   }   /* end of else  */
  
   pthread_setcanceltype(type,NULL);
   pthread_testcancel();
-  return(len - n);
+  return (len - n);
 }                                                                                                    
 /**
  * this function is used for chunked
@@ -497,13 +516,13 @@ socket_readline(CONN *C, char *ptr, size_t maxlen)
   len = maxlen;
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type); 
 
-  for(n = 1; n < len; n ++){
-    if((res = socket_read(C, &c, 1)) == 1){
+  for (n = 1; n < len; n ++) {
+    if ((res = socket_read(C, &c, 1)) == 1) {
       *ptr++ = c;
-      if(c=='\n') break;
+      if (c=='\n') break;
     }
-    else if(res == 0){
-      if(n == 1) 
+    else if (res == 0) {
+      if (n == 1) 
         return 0; 
       else 
         break; 
@@ -536,7 +555,7 @@ socket_write(CONN *C, const void *buf, size_t len)
     return -1;
   }
 #endif
-  if(C->encrypt == TRUE){
+  if (C->encrypt == TRUE) {
     /* handle HTTPS protocol */
     #ifdef HAVE_SSL
     /** 
@@ -544,21 +563,21 @@ socket_write(CONN *C, const void *buf, size_t len)
      * endless loop, (see: Loop, endless), but 
      * a socket timeout will break it...
      */
-    do{
-      if((bytes = __ssl_socket_write(C, buf, len)) != len){
-        if(bytes == 0)
+    do {
+      if ((bytes = __ssl_socket_write(C, buf, len)) != len) {
+        if (bytes == 0)
           ;
         else 
           return -1;
       }
-    } while(bytes == 0);
+    } while (bytes == 0);
     #else
     NOTIFY(ERROR, "%s:%d protocol NOT supported", __FILE__, __LINE__);
     return -1;
     #endif/*HAVE_SSL*/
   } else {
     /* assume HTTP */
-    if((bytes = __socket_write(C->sock, buf, len)) != len){
+    if ((bytes = __socket_write(C->sock, buf, len)) != len) {
       NOTIFY(ERROR, "unable to write to socket %s:%d", __FILE__, __LINE__);
       return -1;
     }
@@ -584,17 +603,17 @@ socket_close(CONN *C)
   int tries = 0;
 #endif/*HAVE_SSL*/
 
-  if(C==NULL) return;
+  if (C==NULL) return;
 
   /* XXX Is this necessary? */ 
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type); 
 
-  if(C->encrypt == TRUE){
+  if (C->encrypt == TRUE) {
 #ifdef  HAVE_SSL
-    if(!C->connection.reuse || C->connection.max == 1){
-      if(C->ssl != NULL){
-        do{
-          if((ret = SSL_shutdown(C->ssl))==1) 
+    if (!C->connection.reuse || C->connection.max == 1){
+      if (C->ssl != NULL) {
+        do {
+          if ((ret = SSL_shutdown(C->ssl))==1) 
             break;
           tries++;
         } while(tries < 5);
@@ -611,13 +630,13 @@ socket_close(CONN *C)
     }
 #endif/*HAVE_SSL*/
   } else {
-    if(C->connection.reuse == 0 || C->connection.max == 1){
-      if(C->sock != -1){
-        if((__socket_block(C->sock, FALSE)) < 0)
+    if (C->connection.reuse == 0 || C->connection.max == 1) {
+      if (C->sock != -1) {
+        if ((__socket_block(C->sock, FALSE)) < 0)
           NOTIFY(ERROR, "unable to set to non-blocking %s:%d", __FILE__, __LINE__);
-        if((C->connection.status > 1)&&(ret = shutdown(C->sock, 2)) < 0)
+        if ((C->connection.status > 1)&&(ret = shutdown(C->sock, 2)) < 0)
           NOTIFY(ERROR, "unable to shutdown the socket %s:%d", __FILE__, __LINE__);
-        if((ret = close(C->sock)) < 0)
+        if ((ret = close(C->sock)) < 0)
           NOTIFY(ERROR, "unable to close the socket %s:%d",    __FILE__, __LINE__);
       }
       C->sock                 = -1;
