@@ -30,10 +30,19 @@ void    parse_time(char *p);
 void    pthread_sleep_np(unsigned int seconds); 
 void    pthread_usleep_np(unsigned long usec); 
 int     pthread_rand_np(unsigned int *ctx); 
+int     urandom();
 BOOLEAN strmatch(char *str1, char *str2);
-void    debug(const char *fmt, ...);
+BOOLEAN okay(int code);
+void    echo(const char *fmt, ...);
+void    debug (const char *fmt, ...);
 char    *uppercase(char *s, size_t len);
 char    *lowercase(char *s, size_t len);
+#ifndef strnlen
+size_t  strnlen(const char *str, size_t len);
+#endif
+#ifndef strncasestr
+char    *strncasestr(const char *str1, const char *str2, size_t len);
+#endif
 
 #endif /*UTIL_H*/
 
